<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2005
sirfragalot@users.sourceforge.net
**********************************************************************************/
/*****
* SELECTRESOURCES class
*
* Select database metadata on a variety of parameters
*****/
class SELECTMETADATA
{
// Constructor
	function SELECTMETADATA($db, $vars)
	{
		$this->db = $db;
		$this->vars = $vars;
		include_once("core/messages/MESSAGES.php");
		$this->messages = new MESSAGES();
		include_once("core/messages/ERRORS.php");
		$this->errors = new ERRORS();
		include_once("core/session/SESSION.php");
		$this->session = new SESSION();
		include_once("core/template/TEMPLATE.php");
		$this->template = new TEMPLATE('content');
		include_once("core/list/LISTCOMMON.php");
		$this->common = new LISTCOMMON($this->db, $this->vars);
		include_once("core/html/MISC.php");
		include_once("core/html/HTML.php");
		$this->html = new HTML();
		include_once("core/html/TABLE.php");
	}
// display form options
	function display($error = FALSE)
	{
//		$this->session->delVar('mywikindx_pagingStart');
		$this->template->setVar('heading', $this->messages->text("heading", "selectMeta"));
///First check, do we have resources?
		$recordset = $this->db->select(array('WKX_database_summary'), array('totalResources'));
		if(!$this->db->fetchOne($recordset))
		{
			$this->template->setVar('body', $this->messages->text('misc', 'noResources'));
			return $this->template->process();
		}
		$this->userBib = $this->session->getVar("mywikindx_bibliography_use");
		$bib = FALSE;
		if($this->userBib)
		{
			$recordset = $this->db->select(array('WKX_user_bibliography'), array('title'), 
				" WHERE " . $this->db->formatfield('id') . "=" . $this->db->tidyInput($this->userBib));
			$row = $this->db->fetchRow($recordset);
			$bib = MISC::p(MISC::span("(" . $this->messages->text("user", "bibliography") . ": " . 
				$row['title'] . ")", "hint"));
		}
		else if($this->session->getVar('setup_multiUser'))
			$bib = MISC::p(MISC::span("(" . $this->messages->text("user", "bibliography") . ": " . 
			$this->messages->text("user", "masterBib") . ")", "hint"));
		include_once("core/html/MISC.php");
		include_once("core/html/TABLE.php");
		$pString = $error ? $error : FALSE;
		$pString .= $bib;
		$pString .= TABLE::tableStart();
		$pString .= TABLE::trStart();
// Types of metadata to search on
		$this->radioButtons = FALSE;
		$fields = array("quote"	=>	$this->messages->text("metadata", "quotes"),
				"quoteComment"	=>	$this->messages->text("metadata", "quoteComments"),
				"paraphrase"	=>	$this->messages->text("metadata", "paraphrases"),
				"paraphraseComment"	=>	$this->messages->text("metadata", "paraphraseComments"),
				"musing"	=>	$this->messages->text("metadata", "musings"),
			);
		$pString .= TABLE::td($this->makeFormMultiple("metadata", $fields));
// Resource types
		$this->grabTypes();
		$pString .= TABLE::td($this->makeFormMultiple("type", $this->type));
// There may not yet be tags in the database.
		$this->grabTags();
		if($this->tag)
			$pString .= TABLE::td($this->makeFormMultiple("tag", $this->tag));
		else
			$pString .= TABLE::td("&nbsp;");
		$pString .= TABLE::trEnd();
		$pString .= TABLE::tableEnd();
		$pString .= TABLE::tdEnd() . TABLE::trEnd() . TABLE::trStart() . TABLE::tdStart() . MISC::hr();
		$pString .= TABLE::tableStart();
		$pString .= TABLE::trStart();
		$this->grabGroups();
		$pString .= TABLE::td($this->makeFormMultiple("category", $this->group));
// There may not yet be keywords in the database.
		$this->grabKeywords();
		if($this->keyword)
			$pString .= TABLE::td($this->makeFormMultiple("keyword", $this->keyword));
		else
			$pString .= TABLE::td("&nbsp;");
		$pString .= TABLE::td($this->userBibs());
		$pString .= TABLE::trEnd();
		$pString .= TABLE::tableEnd();
		$pString .= TABLE::tdEnd() . TABLE::trEnd() . TABLE::trStart() . TABLE::tdStart() . MISC::hr();
		$pString .= TABLE::tableStart();
		$pString .= TABLE::trStart();
		$this->grabCreators();
		if($this->creator)
			$pString .= TABLE::td($this->makeFormMultiple("creator", $this->creator));
		else
			$pString .= TABLE::td("&nbsp;");
		$this->grabPublishers();
		if($this->publisher)
			$pString .= TABLE::td($this->makeFormMultiple("publisher", $this->publisher));
		else
			$pString .= TABLE::td("&nbsp;");
		$this->grabCollections();
		if($this->collection)
			$pString .= TABLE::td($this->makeFormMultiple("collection", $this->collection));
		else
			$pString .= TABLE::td("&nbsp;");
		$pString .= TABLE::trEnd();
		$pString .= TABLE::tableEnd();
		$this->template->setVar('body', $pString);
		return $this->template->process();
	}
// Return select box of user bibliographies for this user if browsing the master bibliography
	function userBibs()
	{
		include_once("core/html/MISC.php");
		include_once("core/html/FORM.php");
		if(!$this->session->getVar("mywikindx_bibliographies"))
			return "&nbsp;";
		$bibs = unserialize($this->session->getVar("mywikindx_bibliographies"));
		if($this->userBib)
			unset($bibs[$this->userBib]);
		if(empty($bibs))
			return "&nbsp;";
		$pString = FORM::formHeader("selectMetaProcess");
		$pString .= FORM::hidden("selectMeta_method", "bibId");
		$selected = $this->session->getVar("selectMeta_bibId");
		if($selected && array_key_exists($selected, $bibs))
			$pString .= FORM::selectedBoxValue($this->messages->text("select", "selectBy", " " . 
			$this->messages->text("select", "notInUserBib")), "selectMeta_bibId", $bibs, $selected, 5);
		else
			$pString .= FORM::selectFBoxValue($this->messages->text("select", "selectBy", " " . 
			$this->messages->text("select", "notInUserBib")), "selectMeta_bibId", $bibs, 5);
		$pString .= MISC::br() . FORM::formSubmit("Select");
		$pString .= FORM::formEnd();
		return $pString;
	}
// make a self-contained form with a multiple select box
	function makeFormMultiple($type, $array)
	{
		include_once("core/html/MISC.php");
		include_once("core/html/FORM.php");
		$pString = FORM::formHeader("selectMetaProcess");
		$pString .= FORM::hidden("selectMeta_method", $type);
		if($selected = $this->session->getVar("selectMeta_" . $type))
		{
			$selectedArray = explode(",", $selected);
			$pString .= FORM::selectedBoxValueMultiple(
				$this->messages->text("select", "selectBy", " " . 
				$this->messages->text("select", $type)), 
				"selectMeta_" . $type, $array, $selectedArray, "5");
		}
		else
			$pString .= FORM::selectFBoxValueMultiple(
				$this->messages->text("select", "selectBy", " " . 
				$this->messages->text("select", $type)), 
				"selectMeta_" . $type, $array, "5");
		$pString .= MISC::br() . $this->messages->text("hint", "multiples") . MISC::br();
		if($this->radioButtons)
			$pString .= $this->radioButtons . MISC::br();
		$pString .= FORM::formSubmit("Select");
		$pString .= FORM::formEnd();
		return $pString;
	}
// get types from database.
	function grabTypes()
	{
		include_once("core/resource/RESOURCEMAP.php");
		$resourceMap = new RESOURCEMAP();
		include_once("core/messages/UTF8.php");
		$this->utf8 = new UTF8();
		foreach($resourceMap->types as $key)
			$this->type[$key] = $this->utf8->encodeUtf8($this->messages->text("resourceType", $key));
		$this->radioButtons = FALSE;
	}
// get tags from database.
	function grabTags()
	{
		include_once("core/file/TAG.php");
		$tag = new TAG($this->db);
		$this->tag = $tag->grabAll();
		$this->radioButtons = FALSE;
	}
// get groups from database.
	function grabGroups()
	{
		include_once("core/html/FORM.php");
		include_once("core/group/GROUP.php");
		$group = new GROUP($this->db);
		$this->group = $group->grabAll();
		$this->makeRadioButtons('category');
	}
// get keywords from database.
	function grabKeywords()
	{
		include_once("core/keyword/KEYWORD.php");
		$keyword = new KEYWORD($this->db);
		$this->keyword = $keyword->grabAll($this->userBib);
		$this->makeRadioButtons('keyword');
	}
// get creators from database.
	function grabCreators()
	{
		include_once("core/html/FORM.php");
		include_once("core/creator/CREATOR.php");
		$creator = new CREATOR($this->db);
		$this->creator = $creator->grabAll($this->userBib);
		$this->makeRadioButtons('creator');
	}
// get publishers from database.
	function grabPublishers()
	{
		include_once("core/html/FORM.php");
		include_once("core/publisher/PUBLISHER.php");
		$publisher = new PUBLISHER($this->db);
		$this->publisher = $publisher->grabAll(FALSE, $this->userBib);
		$this->radioButtons = FALSE;
	}
// get collections from database.
	function grabCollections()
	{
		include_once("core/html/FORM.php");
		include_once("core/collection/COLLECTION.php");
		$collection = new COLLECTION($this->db);
		$this->collection = $collection->grabAll(FALSE, $this->userBib);
		$this->radioButtons = FALSE;
	}
// Create radio buttons for AND and OR
	function makeRadioButtons($type)
	{
		include_once("core/html/FORM.php");
		$type = 'selectMeta_' . $type . 'Method';
		if($this->session->getVar($type) == 'AND')
		{
			$pString = MISC::span(FORM::radioButton(FALSE, $type, 'OR') . " OR", "small") . MISC::br();
			$pString .= MISC::span(FORM::radioButton(FALSE, $type, 'AND', TRUE) . " AND", "small");
		}
// Default
		else
		{
			$pString = MISC::span(FORM::radioButton(FALSE, $type, 'OR', TRUE) . " OR", "small") . MISC::br();
			$pString .= MISC::span(FORM::radioButton(FALSE, $type, 'AND') . " AND", "small");
		}
		$this->radioButtons = $pString;
	}
// Process the selection
	function process($message = FALSE)
	{
		$input = $this->checkInput();
		include_once("core/html/MISC.php");
		$this->template->setVar('heading', $this->messages->text("heading", "selectMeta"));
		$condition = $this->kwq = $this->kwp = $this->kwm = FALSE;
		$this->statementAll();
		switch($input['method'])
		{
			case 'type':
				include_once("core/type/TYPE.php");
				$method = new TYPE($this->db);
				$condition = $method->selectCondition($input);
				break;
			case 'category':
				include_once("core/group/GROUP.php");
				$method = new GROUP($this->db);
				$condition = $method->selectCondition($input);
				break;
			case 'keyword':
				include_once("core/keyword/KEYWORD.php");
				$kw = new KEYWORD($this->db);
				$this->kwq = $kw->selectConditionMetadata($input, 'quote');
				$this->kwp = $kw->selectConditionMetadata($input, 'paraphrase');
				$this->kwm = $kw->selectConditionMetadata($input, 'musing');
				break;
			case 'tag':
				include_once("core/file/TAG.php");
				$method = new TAG($this->db);
				$condition = $method->selectCondition($input);
				break;
			case 'creator':
				include_once("core/creator/CREATOR.php");
				$method = new CREATOR($this->db);
				$condition = $method->selectCondition($input);
				break;
			case 'publisher':
				include_once("core/publisher/PUBLISHER.php");
				$method = new PUBLISHER($this->db, $this->vars, $input);
				$condition = $method->selectCondition($input);
				break;
			case 'collection':
				include_once("core/collection/COLLECTION.php");
				$method = new COLLECTION($this->db, $this->vars, $input);
				$condition = $method->selectCondition($input);
				break;
			case 'bibId':
				$recordSet = $this->db->select(array('WKX_user_bibliography'), array('bibliography'), 
					" WHERE " . $this->db->formatField('WKX_user_bibliography.id') . "=" . 
					$this->db->tidyInput($input['bibId']));
				if($bibliography = $this->db->fetchOne($recordSet))
				{
					$ids = explode(",", $bibliography);
					$condition = $this->db->formatField('WKX_resource.id') ."!=" . 
						join(" AND " . $this->db->formatField('WKX_resource.id') ."!=", $ids);
				}
				break;
			case 'metadata':
				$selectArray = explode(",", $input['metadata']);
//print_r($selectArray); print "<P>";
				if(array_search('quote', $selectArray) === FALSE)
					$this->stmtQuote = FALSE;
				if(array_search('quoteComment', $selectArray) === FALSE)
					$this->stmtQuoteComment = FALSE;
				if(array_search('paraphrase', $selectArray) === FALSE)
					$this->stmtParaphrase = FALSE;
				if(array_search('paraphraseComment', $selectArray) === FALSE)
					$this->stmtParaphraseComment = FALSE;
				if(array_search('musing', $selectArray) === FALSE)
					$this->stmtMusing = FALSE;
				break;
			default:
				$this->badInput($this->errors->text("inputError", "missing", " (selectMeta_method) "));
		}
		if($userBibCondition = $this->common->userBibCondition())
			$userBibCondition = $condition ? " AND " . $userBibCondition : " WHERE " . $userBibCondition;
		$condition = $condition ? " WHERE ($condition)" . $userBibCondition : $userBibCondition;
		$pString = $this->executeStmts($condition);
		$this->session->setVar("sql_lastMultiMeta", 'select');
		$this->template->setVar('body', $pString);
		return $this->template->process();
	}
// Execute each statement and concatenate into string
	function executeStmts($condition)
	{
//print "$condition<P>";
		include_once("core/text/TEXT.php");
		$text = new TEXT($this->db, FALSE);
		include_once("core/user/USER.php");
		$user = new USER($this->db);
		include_once("core/cite/CITE.php");
		$cite = new CITE($this->db, $this->vars);
// Load icons
		include_once("core/icons/LOADICONS.php");
		$icons = new LOADICONS();
		$icons->getIconInfo("view");
		$viewEditLink = $icons->viewExists ? "imgLink" : "link";
		$pString = $conditionQ = $conditionP = $conditionM = $q = $p = $m = FALSE;
		$metadataCountTotal = 0;
		$userId = $this->session->getVar('setup_userId');
		$allIds = array();
		if($this->stmtQuote)
		{
			if($this->kwq)
				$conditionQ = $condition ? "$condition AND (" . $this->kwq . ")" : " WHERE " . $this->kwq;
			else
				$conditionQ = $condition;
			$conditionQ .= " ORDER BY " . $this->db->formatField('resourceId');
//print "$conditionQ<P>";
//print "$this->stmtQuote $conditionQ<P>";
			$index = 0;
			$stmt = $this->stmtQuote . $conditionQ;
			$recordSet = $this->db->query($stmt);
			if($metadataCount = $this->db->numRows($recordSet))
			{
				$q = TABLE::tableStart(FALSE, 0, 0, 0, "left");
				while($row = $this->db->loopRecordSet($recordSet))
				{
					$class = $index % 2 ? 'alternate2' : 'alternate1';
					$q .= $text->dataRow($row['text'], $row['resourceId'], $class);
					if(array_search($row['resourceId'], $allIds) === FALSE)
						$allIds[] = $row['resourceId'];
					$index++;
				}
				$q .= TABLE::tableEnd();
				if($q)
					$pString .= TABLE::tdEnd() . TABLE::trEnd() . TABLE::trStart() . TABLE::tdStart() . 
					"&nbsp;" . MISC::p(MISC::h($this->messages->text("metadata", "quotes", 3))) . $q;
			}
			$metadataCountTotal += $metadataCount;
		}
		if($this->stmtQuoteComment)
		{
			if($this->kwq)
				$conditionQ = $condition ? "$condition AND (" . $this->kwq . ")" : " WHERE " . $this->kwq;
			else
				$conditionQ = $condition;
			$conditionQ .= " ORDER BY " . $this->db->formatField('quoteId');
//print "$conditionQ<P>";
//print "$this->stmtQuoteComment $conditionQ<P>";
			$index = 0;
			$stmt = $this->stmtQuoteComment . $conditionQ;
			$recordSet = $this->db->query($stmt);
			if($metadataCount = $this->db->numRows($recordSet))
			{
				$first = TRUE;
				while($row = $this->db->loopRecordSet($recordSet))
				{
					if(($row['quotePrivate'] == 'Y') && ($userId != $row['addUserIdQuote']))
						continue;
					if($first)
						$q = TABLE::tableStart(FALSE, 0, 0, 0, "left");
					$class = $index % 2 ? 'alternate2' : 'alternate1';
					$username = $user->displayUserAddEdit($row['addUserIdQuote']);
					$q .= TABLE::trStart();
					$q .= TABLE::td($cite->parseCitations($this->html->dbToHtmlTidy($row['comment']) . 
						$username, 'html'), $class);
					$view = MISC::a("$viewEditLink", "&nbsp;" . $icons->view . "&nbsp;", 
						"index.php?action=resourceView" . htmlentities("&id=" . $row['resourceId']));
					$q .= TABLE::td($view, $class, "right");
					$q .= TABLE::trEnd();
					if(array_search($row['resourceId'], $allIds) === FALSE)
						$allIds[] = $row['resourceId'];
					$index++;
					$first = FALSE;
				}
				if($q)
				{
					$q .= TABLE::tableEnd();
					$metadataCountTotal += $metadataCount;
					$pString .= TABLE::tdEnd() . TABLE::trEnd() . TABLE::trStart() . TABLE::tdStart() . 
					"&nbsp;" . MISC::p(MISC::h($this->messages->text("metadata", "quoteComments", 3))) . $q;
				}
			}
		}
		if($this->stmtParaphrase)
		{
			if($this->kwp)
				$conditionP = $condition ? "$condition AND (" . $this->kwp . ")" : " WHERE " . $this->kwp;
			else
				$conditionP = $condition;
			$conditionP .= " ORDER BY " . $this->db->formatField('resourceId');
//print "$conditionP<P>";
//print "$this->stmtParaphrase $conditionP<P>";
			$index = 0;
			$stmt = $this->stmtParaphrase . $conditionP;
			$recordSet = $this->db->query($stmt);
			if($metadataCount = $this->db->numRows($recordSet))
			{
				$p = TABLE::tableStart(FALSE, 0, 0, 0, "left");
				while($row = $this->db->loopRecordSet($recordSet))
				{
					$class = $index % 2 ? 'alternate2' : 'alternate1';
					$p .= $text->dataRow($row['text'], $row['resourceId'], $class);
					if(array_search($row['resourceId'], $allIds) === FALSE)
						$allIds[] = $row['resourceId'];
					$index++;
				}
				$p .= TABLE::tableEnd();
				if($p)
					$pString .= TABLE::tdEnd() . TABLE::trEnd() . TABLE::trStart() . TABLE::tdStart() . 
					"&nbsp;" . MISC::p(MISC::h($this->messages->text("metadata", "paraphrases", 3))) . $p;
			}
			$metadataCountTotal += $metadataCount;
		}
		if($this->stmtParaphraseComment)
		{
			if($this->kwp)
				$conditionP = $condition ? "$condition AND (" . $this->kwp . ")" : " WHERE " . $this->kwp;
			else
				$conditionP = $condition;
			$conditionP .= " ORDER BY " . $this->db->formatField('paraphraseId');
//print "$conditionP<P>";
//print "$this->stmtParaphraseComment $conditionP<P>";
			$index = 0;
			$stmt = $this->stmtParaphraseComment . $conditionP;
			$recordSet = $this->db->query($stmt);
			if($metadataCount = $this->db->numRows($recordSet))
			{
				$first = TRUE;
				while($row = $this->db->loopRecordSet($recordSet))
				{
					if(($row['paraphrasePrivate'] == 'Y') && ($userId != $row['addUserIdParaphrase']))
						continue;
					if($first)
						$p = TABLE::tableStart(FALSE, 0, 0, 0, "left");
					$class = $index % 2 ? 'alternate2' : 'alternate1';
					$username = $user->displayUserAddEdit($row['addUserIdParaphrase']);
					$p .= TABLE::trStart();
					$p .= TABLE::td($cite->parseCitations($this->html->dbToHtmlTidy($row['comment']) . 
						$username, 'html'), $class);
					$view = MISC::a("$viewEditLink", "&nbsp;" . $icons->view . "&nbsp;", 
						"index.php?action=resourceView" . htmlentities("&id=" . $row['resourceId']));
					$p .= TABLE::td($view, $class, "right");
					$p .= TABLE::trEnd();
					if(array_search($row['resourceId'], $allIds) === FALSE)
						$allIds[] = $row['resourceId'];
					$index++;
					$first = FALSE;
				}
				if($p)
				{
					$p .= TABLE::tableEnd();
					$metadataCountTotal += $metadataCount;
					$pString .= TABLE::tdEnd() . TABLE::trEnd() . TABLE::trStart() . TABLE::tdStart() . 
					"&nbsp;" . MISC::p(MISC::h($this->messages->text("metadata", 
					"paraphraseComments", 3))) . $p;
				}
			}
		}
		if($this->stmtMusing)
		{
			if($this->kwm)
				$conditionM = $condition ? "$condition AND (" . $this->kwm . ")" : " WHERE " . $this->kwm;
			else
				$conditionM = $condition;
			$conditionM .= " ORDER BY " . $this->db->formatField('resourceId');
//print "$conditionM<P>";
//print "$this->stmtMusing $conditionM<P>";
			$index = 0;
			$stmt = $this->stmtMusing . $conditionM;
			$recordSet = $this->db->query($stmt);
			if($metadataCount = $this->db->numRows($recordSet))
			{
				$first = TRUE;
				while($row = $this->db->loopRecordSet($recordSet))
				{
					if(($row['musingPrivate'] == 'Y') && ($userId != $row['addUserIdMusing']))
						continue;
					if($first)
						$m = TABLE::tableStart(FALSE, 0, 0, 0, "left");
					$class = $index % 2 ? 'alternate2' : 'alternate1';
					$m .= $text->dataRow($row['text'], $row['resourceId'], $class);
					if(array_search($row['resourceId'], $allIds) === FALSE)
						$allIds[] = $row['resourceId'];
					$index++;
					$first = FALSE;
				}
				if($m)
				{
					$m .= TABLE::tableEnd();
					$metadataCountTotal += $metadataCount;
					$pString .= TABLE::tdEnd() . TABLE::trEnd() . TABLE::trStart() . TABLE::tdStart() . 
					"&nbsp;" . MISC::p(MISC::h($this->messages->text("metadata", "musings", 3))) . $m;
				}
			}
		}
		if(!$pString)
			return $this->messages->text("metadata", "noResultMeta");
// Write resource ids found here to session for use with previous and next icons when viewing a single resource.
		if(!empty($allIds))
			$this->session->setVar('list_allIds', base64_encode(serialize($allIds)));
		else
			$this->session->delVar("list_allIds");
// Get name of current bibliography
		$useBib = $this->session->getVar('mywikindx_bibliography_use');
		if($useBib)
		{
			$recordset = $this->db->select(array('WKX_user_bibliography'), array('title'), 
				" WHERE " . $this->db->formatfield('id') . "=" . $this->db->tidyInput($useBib));
			$row = $this->db->fetchRow($recordset);
			$bib = " (" . $this->messages->text("user", "bibliography") . ": " . 
				$this->html->dbToHtmlTidy($row['title']) . ")";
		}
		else
			$bib = " (" . $this->messages->text("user", "bibliography") . ": " . 
				$this->messages->text("user", "masterBib") . ")";
		return MISC::p($this->messages->text("hint", "displayMeta", 
			' ' . $metadataCountTotal . $bib)) . $pString;
	}
// Create basic select statement for all metadata tables
	function statementAll()
	{
		$this->stmtQuote = $this->db->selectNoExecute(array('WKX_resource_quote'), 
			array('resourceId', 'text'), 
			" LEFT JOIN " . $this->db->formatTable('WKX_resource_quote_text') . " ON " . 
			$this->db->formatfield("WKX_resource_quote_text.id") . "=" . 
			$this->db->formatfield("WKX_resource_quote.id"));
		$this->stmtQuoteComment = $this->db->selectNoExecute(array('WKX_resource_quote_comment'), 
			array('resourceId', 'quoteId', 'comment', 'quotePrivate',
				'WKX_resource_quote_comment.addUserIdQuote'), 
			" LEFT JOIN " . $this->db->formatTable('WKX_resource_quote') . " ON " . 
			$this->db->formatfield("quoteId") . "=" . 
			$this->db->formatfield("WKX_resource_quote.id"));
		$this->stmtParaphrase = $this->db->selectNoExecute(array('WKX_resource_paraphrase'), 
			array('resourceId', 'text'), 
			" LEFT JOIN " . $this->db->formatTable('WKX_resource_paraphrase_text') . " ON " . 
			$this->db->formatfield("WKX_resource_paraphrase_text.id") . "=" . 
			$this->db->formatfield("WKX_resource_paraphrase.id"));
		$this->stmtParaphraseComment = $this->db->selectNoExecute(array('WKX_resource_paraphrase_comment'), 
			array('resourceId', 'paraphraseId', 'comment', 'paraphrasePrivate',
				'WKX_resource_paraphrase_comment.addUserIdParaphrase'), 
			" LEFT JOIN " . $this->db->formatTable('WKX_resource_paraphrase') . " ON " . 
			$this->db->formatfield("paraphraseId") . "=" . 
			$this->db->formatfield("WKX_resource_paraphrase.id"));
		$this->stmtMusing = $this->db->selectNoExecute(array('WKX_resource_musing'), 
			array('resourceId', 'text', 'addUserIdMusing', 'musingPrivate'), 
			" LEFT JOIN " . $this->db->formatTable('WKX_resource_musing_text') . " ON " . 
			$this->db->formatfield("WKX_resource_musing_text.id") . "=" . 
			$this->db->formatfield("WKX_resource_musing.id"));
		$extraJoin = " LEFT JOIN " . $this->db->formatTable('WKX_resource') . " ON " . 
			$this->db->formatfield("resourceId") . "=" . 
			$this->db->formatfield("WKX_resource.id") .  
			" LEFT JOIN " . $this->db->formatTable('WKX_resource_creator') . " ON " . 
			$this->db->formatfield("resourceId") . "=" . 
			$this->db->formatfield("WKX_resource_creator.id") . 
			" LEFT JOIN " . $this->db->formatTable('WKX_resource_misc') . " ON " . 
			$this->db->formatfield("resourceId") . "=" . 
			$this->db->formatfield("WKX_resource_misc.id") . 
			" LEFT JOIN " . $this->db->formatTable('WKX_resource_category') . " ON " . 
			$this->db->formatfield("resourceId") . "=" . 
			$this->db->formatfield("WKX_resource_category.id");
		$this->stmtQuote .= $extraJoin;
		$this->stmtQuoteComment .= $extraJoin;
		$this->stmtParaphrase .= $extraJoin;
		$this->stmtParaphraseComment .= $extraJoin;
		$this->stmtMusing .= $extraJoin;
	}
// write input to session
	function writeSession()
	{
// If this is a different type of select to the previous one, reset the paging counter.
// Write all input with 'selectMeta_' prefix to session
		foreach($this->vars as $key => $value)
		{
			if(preg_match("/^selectMeta_/", $key))
			{
// Is this a multiple select box input?  If so, multiple choices are written to session as 
// comma-delimited string (no spaces).
				if(is_array($value))
					$value = implode(",", $value);
				$temp[$key] = $value;
//				if($this->session->getVar("selectMeta_$key") != $value)
//					$this->session->delVar('mywikindx_pagingStart');
			}
		}
		if(isset($temp))
			$this->session->writeArray($temp);
	}
// validate user input
// Input comes either from form input or from the session.
	function checkInput()
	{
		if(array_key_exists("selectMeta_method", $this->vars) && $this->vars["selectMeta_method"])
			$type = $this->vars["selectMeta_method"];
		else if($this->session->issetVar('selectMeta_method'))
			$type = $this->session->getVar("selectMeta_method");
		else
			$this->badInput($this->errors->text("inputError", "missing", " (selectMeta_method) "));
		if(!array_key_exists("selectMeta_" . $type, $this->vars) &&  
			!$this->session->issetVar("selectMeta_" . $type))
			$this->badInput($this->errors->text("inputError", "missing", " (selectMeta_" . $type . ") "));
// If missing the following, set to default 'OR'.
		if(!array_key_exists("selectMeta_" . $type . "Method", $this->vars) &&
		!$this->session->issetVar("selectMeta_" . $type . "Method"))
			$this->vars["selectMeta_" . $type . "Method"] = 'OR';
		$this->writeSession();
		return $this->session->getArray("selectMeta");
	}
// Error handling
	function badInput($error)
	{
		include_once("core/html/CLOSE.php");
		new CLOSE($this->db, $this->display($error));
	}
}
?>
